using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Syncfusion.PivotAnalysis.Base;
using Syncfusion.Windows.Forms.PivotChart;
using Syncfusion.Windows.Forms;
using System.Reflection;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.WinForms.Controls;
using System.Collections.Specialized;

namespace PivotChartTypes_Demo
{
    public partial class FieldList : SfForm
    {
        private bool IsUpdateOK = false;
        private bool IsFilterAdd = false;
        private BackgroundWorker backgroundWorker = new BackgroundWorker();
        private Dictionary<string, string> FilterExpressions = new Dictionary<string, string>();

        public FieldList()
        {
            InitializeComponent();
            backgroundWorker.WorkerReportsProgress = true;
            backgroundWorker.DoWork += BackgroundWorker_DoWork;
            backgroundWorker.RunWorkerCompleted += BackgroundWorker_RunWorkerCompleted;

            try
            {
                System.Drawing.Icon ico = new System.Drawing.Icon(GetIconFile(@"common\Images\Grid\Icon\sfgrid.ico"));
                this.Icon = ico;
            }
            catch { }

            this.pivotChart1 = new PivotChart();
            this.pivotChart1.Dock = DockStyle.Fill;
            this.chart.Controls.Add(this.pivotChart1);

            pivotChart1.ItemSource = ProductSales.GetSalesData();

            pivotChart1.PivotAxis.Add(new PivotItem
            {
                FieldMappingName = nameof(ProductSales.Product)
            });
            pivotChart1.PivotLegend.Add(new PivotItem
            {
                FieldMappingName = nameof(ProductSales.Country)
            });
            pivotChart1.PivotLegend.Add(new PivotItem
            {
                FieldMappingName = nameof(ProductSales.State)
            });
            pivotChart1.PivotCalculations.Add(new PivotComputationInfo
            {
                FieldName = nameof(ProductSales.Quantity)
            });

            pivotChart1.ShowPivotTableFieldList = true;
            pivotChart1.DeferLayoutUpdate = true;
            pivotChart1.PivotSchemaDesigner.DeferLayoutUpdateCheckBox.Visible = false;
            pivotChart1.PivotSchemaDesigner.DeferLayoutUpdateButton.Visible = false;

            pivotChart1.ShowLegend = true;
            pivotChart1.ChartControl.ShowContextMenu = true;
            pivotChart1.ChartControl.ShowContextMenuInLegend = true;

            pivotChart1.PivotEngine.PivotSchemaChanged += PivotEngine_PivotSchemaChanged;
            pivotChart1.PivotLegend.CollectionChanged += PivotLegend_CollectionChanged;
            pivotChart1.PivotAxis.CollectionChanged += PivotAxis_CollectionChanged;
            pivotChart1.PivotCalculations.CollectionChanged += PivotCalculations_CollectionChanged;
            pivotChart1.PivotFilters.CollectionChanged += PivotFilters_CollectionChanged;
        }
        private void BackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.pictureBox.Visible = false;
            this.IsUpdateOK = false;
            this.IsFilterAdd = false;
            pivotChart1.BringToFront();
            pivotChart1.ReloadChartData();
        }

        private void BackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            pivotChart1.PivotEngine.Populate();
        }

        private void PivotEngine_PivotSchemaChanged(object sender, PivotSchemaChangedArgs e)
        {
            foreach (FilterExpression fexpression in pivotChart1.PivotFilters)
            {
                if (this.IsFilterAdd)
                {
                    break;
                }
                if (this.FilterExpressions.ContainsKey(fexpression.DimensionName))
                {
                    var expression = this.FilterExpressions[fexpression.DimensionName];
                    if (expression != fexpression.Expression)
                    {
                        this.IsUpdateOK = true;
                        break;
                    }
                }
            }

            if (this.IsUpdateOK)
            {
                this.AsyncUpdate();
            }
        }

        private void PivotLegend_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            this.AsyncUpdate();
        }

        private void PivotAxis_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            this.AsyncUpdate();
        }

        private void PivotCalculations_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            this.AsyncUpdate();
        }

        private void PivotFilters_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            // Remove
            if (e.Action == NotifyCollectionChangedAction.Remove)
            {
                this.AsyncUpdate();
                return;
            }
            this.IsUpdateOK = true;
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                var fexpression = e.NewItems[0] as FilterExpression;
                if (!this.FilterExpressions.ContainsKey(fexpression.DimensionName))
                {
                    this.FilterExpressions.Add(fexpression.DimensionName, fexpression.Expression);
                    this.IsFilterAdd = true;
                }
            }
        }

        private void AsyncUpdate()
        {
            if (!backgroundWorker.IsBusy)
            {
                this.pictureBox.Visible = true;
                this.pictureBox.BringToFront();
                this.Refresh();
                backgroundWorker.RunWorkerAsync();
            }
        }

            #region "ICON FILE"

            private string GetIconFile(string bitmapName)
        {
            for (int n = 0; n < 10; n++)
            {
                if (System.IO.File.Exists(bitmapName))
                    return bitmapName;

                bitmapName = @"..\" + bitmapName;
            }

            return bitmapName;
        }

        #endregion

        #region "DATASOURCE"

        public class ProductSales
        {
            public string Product { get; set; }

            public string Date { get; set; }

            public string Country { get; set; }

            public string State { get; set; }

            public int Quantity { get; set; }

            public double Amount { get; set; }

            public double UnitPrice { get; set; }

            public double TotalPrice { get; set; }

            public static ProductSalesCollection GetSalesData()
            {
                /// Geography
                string[] countries = new string[] { "Australia","Canada", "Germany", "United States" };
                string[] ausStates = new string[] { "New South Wales", "Queensland", };
                string[] canadaStates = new string[] { "Saskatchewan", "Quebec" };
                string[] germanyStates = new string[] { "Bayern", "Brandenburg" };
                string[] ussStates = new string[] {"New Mexico" };

                /// Time
                string[] dates = new string[] { "FY 2008", "FY 2009", "FY 2010", "FY 2011", "FY 2012" };

                /// Products
                string[] products = new string[] { "Bike" };
                Random r = new Random(123345);

                int numberOfRecords = 2000;
                ProductSalesCollection listOfProductSales = new ProductSalesCollection();
                for (int i = 0; i < numberOfRecords; i++)
                {
                    ProductSales sales = new ProductSales();
                    sales.Country = countries[r.Next(1, countries.GetLength(0))];
                    sales.Quantity = r.Next(1, 12);
                    /// 1 percent discount for 1 quantity
                    double discount = (30000 * sales.Quantity) * (double.Parse(sales.Quantity.ToString()) / 100);
                    sales.Amount = (30000 * sales.Quantity) - discount;
                    sales.TotalPrice = sales.Amount * sales.Quantity;
                    sales.UnitPrice = sales.Amount / sales.Quantity;
                    sales.Date = dates[r.Next(r.Next(dates.GetLength(0) + 1))];
                    sales.Product = products[r.Next(r.Next(products.GetLength(0) + 1))];
                    switch (sales.Country)
                    {
                        case "Australia":
                            {
                                sales.State = ausStates[r.Next(ausStates.GetLength(0))];
                                break;
                            }
                        case "Canada":
                            {
                                sales.State = canadaStates[r.Next(canadaStates.GetLength(0))];
                                break;
                            }
                        case "Germany":
                            {
                                sales.State = germanyStates[r.Next(germanyStates.GetLength(0))];
                                break;
                            }
                        case "United States":
                            {
                                sales.State = ussStates[r.Next(ussStates.GetLength(0))];
                                break;
                            }
                    }
                    listOfProductSales.Add(sales);
                }

                return listOfProductSales;
            }

            public override string ToString()
            {
                return string.Format("{0}-{1}-{2}", this.Country, this.State, this.Product);
            }

            public class ProductSalesCollection : List<ProductSales>
            {
            }
        }
        #endregion

        private PivotChart pivotChart1;
    }
}
